'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
var fs = require('fs');
var path = require('path');
var md5File = require('md5-file');
var mimeTypes = require('mime-types');
var register = require('ignore-styles').default;

var createSSRRegisterHook = exports.createSSRRegisterHook = function createSSRRegisterHook() {
    var settings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var importExtensions = settings.ext || ['.png', '.jpg', 'jpeg', '.gif', 'svg'];
    var importLimit = settings.limit || 10000;

    register(undefined, function (mod, filename) {
        try {
            var ext = importExtensions.find(function (f) {
                return filename.endsWith(f);
            });
            if (!ext) {
                return;
            }

            // embed image as base64
            if (fs.statSync(filename).size < importLimit) {
                var file = fs.readFileSync(filename).toString('base64');
                var mimeType = mimeTypes.lookup(ext) || 'image/jpg';
                mod.exports = 'data:' + mimeType + ';base64,' + file;

                // link image to the media folder
            } else {
                var hash = md5File.sync(filename).slice(0, 8);
                var bn = path.basename(filename).replace(/(\.\w{3})$/, '.' + hash + '$1');
                mod.exports = '/static/media/' + bn;
            }
        } catch (err) {
            console.log('[ssr] ignore-styles :: ' + err.message);
            console.log(err);
        }
    });
};